/*
 * TRESOR - int0x13 hook
 *
 * Copyright (C) 2013 Johannes Goetzfried <johannes@jgoetzfried.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 */

#include "constants.h"
#include "initfunc.h"
#include "mm.h"
#include "string.h"
#include "tresor_int0x13_hook.h"

static void
install_int0x13_hook (void)
{
	u64 int0x13_code, int0x13_base;
	u64 int0x13_vector_phys = 0x13 * 4;
	int len;
	void *p;

	len = guest_int0x13_hook_end - guest_int0x13_hook;
	int0x13_code = alloc_realmodemem (len);
	int0x13_base = int0x13_code & 0xFFFF0;

	/* save old interrupt vector */
	read_hphys_l (int0x13_vector_phys, &guest_int0x13_orig, 0);

	/* copy the program code */
	p = mapmem_hphys (int0x13_code, len, MAPMEM_WRITE);
	memcpy (p, guest_int0x13_hook, len);
	unmapmem (p, len);

	/* set interrupt vector */
	write_hphys_l (int0x13_vector_phys, (int0x13_code - int0x13_base) |
		       (int0x13_base << 12), 0);
}

#ifdef TRESOR
INITFUNC ("bsp0", install_int0x13_hook);
#endif
