#!/bin/bash

set -e -u

usage() {
	cat >&2 <<EOF
Usage: $(basename $0) <device>
Prepare USB Stick for MARK. Make sure to use the right device name!

Example: $(basename $0) /dev/sdf
EOF
}


[ $# -eq 1 ] || { usage; exit 0; }

DEVICE="$1"
[ -b "$DEVICE" ] || { echo "$DEVICE is no valid block device"; exit 1; }
[ `whoami` = "root" ] || { echo "You have to be root!"; exit 1; }

fdisk $DEVICE <<EOF
o
n
p
1
2048
+256MB
n
p
2

+256MB
w
EOF

mkfs.ext3 -L MARK_SYSTEM "$DEVICE"1
mkfs.ext3 -L MARK_DATA "$DEVICE"2

mkdir /tmp/mnttmp-$$
mount "$DEVICE"1 /tmp/mnttmp-$$
mkdir /tmp/mnttmp-$$/grub
cp $(dirname $0)/tgrub/stage? /tmp/mnttmp-$$/grub
cp $(dirname $0)/menu.lst /tmp/mnttmp-$$/grub

# These files need to be build separately
cp $(dirname $0)/../bitvisor/bitvisor.elf /tmp/mnttmp-$$/
cp $(dirname $0)/../kernel/vmlinux /tmp/mnttmp-$$/
cp $(dirname $0)/../debirf/mark/mark_wheezy.cgz /tmp/mnttmp-$$/

umount "$DEVICE"1
rmdir /tmp/mnttmp-$$

$(dirname $0)/tgrub/grub <<EOF
device (hd0) $DEVICE
root (hd0,0)
setup (hd0)
quit
EOF

exit 0
