/*
 * minios_crypt - AES XTS encryption in userspace
 *
 * Copyright (C) 2013 Johannes Goetzfried <johannes@jgoetzfried.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include "aes_xts.h"

int main(int argc, char **argv) {
	uint8_t dek[32];
	const char *dst, *src;
	FILE *fdst, *fsrc;
	uint64_t sectorcnt;
	uint32_t sectorsize = 512;
	uint8_t *buffer;
	struct aes_xts_keyctx *ctx;
	size_t ret;

	if (argc < 4) {
		fprintf(stderr, "Usage: %s <dst> <src> <n> [sectorsize]\n", *argv);
		fprintf(stderr, "Read <n> sectors from <src> and write encrypted result to <dst>\n");
		fprintf(stderr, "Default sectorsize: 512 Bytes\n");
		return EXIT_FAILURE;
	}

	dst = argv[1];
	src = argv[2];
	sectorcnt = atoll(argv[3]);
	if (argc > 4)
		sectorsize = atoi(argv[4]);

	fdst = fopen(dst, "w");
	fsrc = fopen(src, "r");

	if (!fdst || !fsrc) {
		perror("Could not open devices!\n");
		return EXIT_FAILURE;
	}

	buffer = malloc(sectorsize * sizeof(uint8_t));
	if (buffer == NULL) {
		fprintf(stderr, "Memory allocation failure!\n");
		return EXIT_FAILURE;
	}

	/* read the dek from stdin */
	ret = fread(dek, 32, 1, stdin);
	if (ret != 1) {
		fprintf(stderr, "Could not read DEK!\n");
		return EXIT_FAILURE;
	}

	ctx = aes_xts_setkey(dek, 256);

	for (uint64_t s = 0; s < sectorcnt; s++) {
		ret = fread(buffer, sectorsize, 1, fsrc);
		if (ret != 1)
			break;

		aes_xts_encrypt(buffer, buffer, ctx, s, sectorsize);

		ret = fwrite(buffer, sectorsize, 1, fdst);
		if (ret != 1) {
			fprintf(stderr, "No space left on destination device!\n");
			return EXIT_FAILURE;
		}
	}

	free_aes_xts_keyctx(ctx);
	free(buffer);
	fclose(fsrc);
	fclose(fdst);

	return EXIT_SUCCESS;
}
