# MARK - Crypto Routines
#
# Copyright (C) 2013  Johannes Goetzfried <johannes@jgoetzfried.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import string;

from Crypto.Protocol.KDF import PBKDF2;
from Crypto.Hash import HMAC, SHA256;
from Crypto.Cipher import AES;
from Crypto.Random import random;
from Crypto import Random;

def derive_key(passwd, salt, klen = 32):
	prf = lambda p, s: HMAC.new(p, s, SHA256).digest();
	return PBKDF2(passwd, salt, klen, 4096, prf);

def hash_passwd(passwd, salt):
	return SHA256.new(passwd + salt).digest();

def get_random_bytes(count):
	return Random.new().read(count);

def get_random_alphanum(count):
	return ''.join(random.choice(string.ascii_letters + string.digits) for x in range(count));

class Cipher:
	def __init__(self, key, iv):
		self._cipher = AES.new(key, AES.MODE_CBC, iv);

	def encrypt(self, plaintext):
		return self._cipher.encrypt(plaintext);

	def decrypt(self, ciphertext):
		return self._cipher.decrypt(ciphertext);

