# MARK - User Interface Core
#
# Copyright (C) 2013  Johannes Goetzfried <johannes@jgoetzfried.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import locale;
import curses;
import curses.panel;

# UTF-8 symbols
#CORN_TL = u'\u2554'.encode('utf_8');
#CORN_TR = u'\u2557'.encode('utf_8');
#CORN_BL = u'\u255a'.encode('utf_8');
#CORN_BR = u'\u255d'.encode('utf_8');
#EDGE_V  = u'\u2551'.encode('utf_8');
#EDGE_H  = u'\u2550'.encode('utf_8');
#T_VL    = u'\u2563'.encode('utf_8');
#T_VR    = u'\u2560'.encode('utf_8');

# ASCII symbols
T_VL    = '[';
T_VR    = ']';

_ui = None;

def ui():
	global _ui
	if _ui == None:
		_ui = UI();

	return _ui


class UI:
	def __init__(self):
		self._screen = None;

		locale.setlocale(locale.LC_ALL,'');
		self._screen = curses.initscr();
		curses.start_color();
		curses.noecho();
		curses.cbreak();
		curses.curs_set(0);
		self._screen.clear();
		self._screen.keypad(1);

	def update(self):
		curses.panel.update_panels();
		curses.doupdate();

	def gethw(self):
		return self._screen.getmaxyx();

	def getch(self):
		return self._screen.getch();

	def handlekey(self, ch):
		# TODO handle resize events
		pass

	def end(self):
		self.update();
		self._screen.refresh();
		curses.endwin();


class Panel:
	def __init__(self, h, w, y, x, border=True, title=''):
		self.h = h;
		self.w = w;
		self.y = y;
		self.x = x;

		self._win   = curses.newwin(h, w, y, x);
		self._panel = curses.panel.new_panel(self._win);

		if border:
			self.border();

		self.title(title);

	def move(self, y, x):
		self._panel.move(y, x);
		self.y = y;
		self.x = x;

	def hide(self):
		self._panel.hide();

	def erase(self):
		self._win.erase();

	def clear(self):
		self._win.clear();

	def go(self, y, x):
		self._win.move(y, x);

	def gethw(self):
		return self.h, self.w;

	def getyx(self):
		return self._win.getyx();

	def getscreenyx(self):
		y, x = self.getyx();
		return y + self.y, x + self.x;

	def write(self, string, lines=1):
		y, x = self.getyx();
		self.writeto(y, x, string, lines);

	def writeto(self, y, x, string, lines=1):
		l = string.encode('utf-8').split('\n');
		for i in l:
			self._win.addnstr(y + l.index(i), x, i, lines * self.w - x - 1);

	def background(self, fg, bg):
		self._win.bkgd(_get_color_attr(fg, bg));

	def fg(self, fg):
		attr = self._win.getbkgd();
		d, bg = curses.pair_content(curses.pair_number(attr));
		self._win.attron(_get_color_attr(fg, bg));

	def bg(self, bg):
		attr = self._win.getbkgd();
		fg, d = curses.pair_content(curses.pair_number(attr));
		self._win.attron(_get_color_attr(fg, bg));

	def border(self):
		# try:
		# 	self._win.addstr(0, 0, CORN_TL);
		# 	for i in range(1, self.w-1):
		# 		self._win.addstr(0, i, EDGE_H);
		# 	self._win.addstr(0, self.w-1, CORN_TR);
		# 	for i in range(1, self.h-1):
		# 		self._win.addstr(i, 0, EDGE_V);
		# 	for i in range(1, self.h-1):
		# 		self._win.addstr(i, self.w-1, EDGE_V);
		# 	self._win.addstr(self.h-1, 0, CORN_BL);
		# 	for i in range(1, self.w-1):
		# 		self._win.addstr(self.h-1, i, EDGE_H);
		# 	self._win.addstr(self.h-1, self.w-1, CORN_BR);
		# except:
		# 	pass
		self._win.border();

	def title(self, title):
		if title == '':
			return;
		p = (self.w - len(title) - 4) / 2;
		self._win.addstr(0, p, T_VL + " " + title.encode('utf-8') + " " + T_VR);


class CenterPanel(Panel):
	def __init__(self, h, w, title=''):
		sh, sw = ui().gethw();
		y = (sh - h) / 2;
		x = (sw - w) / 2;
		Panel.__init__(self, h, w, y, x, True, title);


def _get_color_attr(fg, bg):
	if (fg, bg) in _get_color_attr._colormap:
		return curses.color_pair(_get_color_attr._colormap[(fg, bg)]);

	num = _get_color_attr._current_pair_number;
	curses.init_pair(num, fg, bg);
	_get_color_attr._current_pair_number = _get_color_attr._current_pair_number + 1;
	_get_color_attr._colormap[(fg, bg)] = num;
	return curses.color_pair(num);

_get_color_attr._colormap = {};
_get_color_attr._current_pair_number = 1;

